
function [Mean_Residuals,Max_Residuals, Plot_Residuals]  = Residuals_L3(k,z,A,alpha,lss,iss,phi,gam,nu,B,delta,sigma,rho,beta,vf,pk,D)

T     = size(k,1);   % Simulation length

% Gauss Hermite quadrature integration in the accuracy evaluation procedure:
% the number of integration nodes, their values and weights
%--------------------------------------------------------------------------
Qn = 10;             % Number of unidimensional integration nodes 
nshocks = 1;         % There is one stochastic shock
vcv = sigma^2;       % Variance covariance matrix
[n_nodes,epsi_nodes,weight_nodes] = GH_Quadrature(Qn,nshocks,vcv);
                     % Gauss Hermite quadrature nodes and weights;
                     % With one shock, n_nodes = Qn
                     
 for t=1:T;   
     
     k0 = k(t,1);         % Capital of period t
     z0 = z(t,1);         % Productivity level of period t 
 
     % Current period quantities in (k0,a0)
     % ------------------------------------     
     Vder0= Polynomial_deriv_2d([k0 z0],D)*vf;  

                                  % Construct the derivative of value function
     k1 = Polynomial_2d([k0 z0],D)*pk;  k1 = max(k1,k0*(1-delta)+phi*iss);
                                    %construct the policy function
     n0=csolve('Labor_Newton',1-lss,[],0.000001,5,nu,gam,alpha,delta,B,A,k0,z0,k1); n0 = min(0.95, n0);
     n0 = max(0.01, n0);
                                   % Solve for labor 
     c0 = (1-delta)*k0+A*z0.*k0.^alpha.*n0.^(1-alpha)-k1;
                                   % Compute consumption using resource
                                   % constraint
       
                                   % Compute next-period capital using budget 
                                   % constraint (2) in MM(2013)
   
     % Future period quantities in n_nodes integration nodes (k1,a1)
     % -------------------------------------------------------------     
     z1 = z0.^rho.*exp(epsi_nodes); % Productivity in n_nodes nodes
     z2 = z1.^rho.*exp(epsi_nodes);
     k1_dupl = ones(n_nodes,1)*k1;  % k1 is the same in n_nodes nodes       
     Vder1(1:n_nodes,1) =  Polynomial_deriv_2d([k1_dupl z1(:,1)],D)*vf;   
     
                                    % Construct the next period derivative
                                    % of value function in the integration
                                    % nodes
     for j=1:n_nodes           
        k2(j,1) = max(Polynomial_2d([k1_dupl(j,1) z1(j,1)],D)*pk,k1_dupl(j,1)*(1-delta)+phi*iss);
        Vder2(j,1) =  Polynomial_deriv_2d([k2(j,1) z2(j,1)],D)*vf;   

        n1(j,1)=csolve('Labor_Newton',n0,[],0.000001,5,nu,gam,alpha,delta,B,A,k1_dupl(j,1),z1(j,1),k2(j,1)); 
        n1(j,1) = min(0.95, n1(j,1));   n1(j,1) = max(0.01, n1(j,1));
       
        c1(j,1) = (1-delta)*k1_dupl(j,1)+A*z1(j,1).*k1_dupl(j,1).^alpha.*n1(j,1).^(1-alpha)-k2(j,1);
      
        lambda1(j,1)=c1(j,1).^(-gam)-beta.*Vder2(j,1);
        lambda1(j,1)=max(lambda1(j,1),0);
      end

       lambda0=  c0^(-gam)-beta*weight_nodes'*Vder1; lambda0=max(lambda0,0);
     % Unit-free residuals in the Euler equation
     % -----------------------------------------
     Residuals(t,1) = weight_nodes'*(beta*c1(1:n_nodes,1).^(-gam).*(1-delta+alpha*A*z1(1:n_nodes,1).*k1.^(alpha-1).*n1(1:n_nodes,1).^(1-alpha)-beta*(1-delta)*lambda1(1:n_nodes,1)))./(c0.^(-gam)-lambda0)-1;
     lambdas0(t,1)=lambda0; lambdas1(t,1)=weight_nodes'*lambda1;
 end

 % Output of the accuracy evaluation test
 % --------------------------------------
 Mean_Residuals = log10(mean(mean(abs(Residuals(1:end))))); % Mean residuals 
 Max_Residuals = log10(max(max(abs(Residuals(1:end)))));    % Maximum residuals
 Plot_Residuals = log10(abs(Residuals(1:end)));
    end