
clear all; clc;
    
global delta alpha beta theta rho sigma grid_k  n_z  i_ss value_ss

do_stat=0; % select 1 if you would like to compute statistics along the simulated path

%----------------------------------------------------------------
% 1. Steady state
%----------------------------------------------------------------
%Utility function: theta ln(c) + (1-theta)ln(1-labor)
%Load parameters
parameters;

AA     =(1/alpha*(1/beta-1+delta))^(1/(1-alpha));
BB   =(AA^(1-alpha)-delta);
CC = theta/(1-theta)*(1-alpha)*AA^(-alpha);

k_ss = CC/(BB+AA*CC);
l_ss = AA*k_ss;
c_ss = BB*k_ss;
r_ss = alpha*(k_ss^(alpha-1))*(l_ss^(1-alpha));
y_ss = k_ss^alpha*(l_ss^(1-alpha));
i_ss = delta*k_ss;


value_ss =theta*log(c_ss)+(1-theta)*log(1-l_ss);

%----------------------------------------------------------------
% 2. Productivity shock process
%----------------------------------------------------------------
n_z = 9;								

[y,transition, transitioncdf]=rouwenhorst(rho,sigma,n_z); 

% ----------------------------------------------------------------
% 3. Form the grid for k
%------------------------------------------------------------------

n_k = 500; 
kmax=k_ss*1.8; kmin=k_ss*.3;
grid_k=linspace(kmin,kmax,n_k);
step0=grid_k(2)-grid_k(1);

time1=cputime;

%----------------------------------------------------------------
% 4. PFI
%----------------------------------------------------------------

[g_k,g_c,g_l,v,lambda]=sub_pfi(grid_k,n_k,n_z,y,transition,step0);

time2=cputime;
cpuseconds=time2-time1;

disp('CPU time(s):');
disp(cpuseconds);

%----------------------------------------------------------------
% 5. Accuracy
%----------------------------------------------------------------
do_stat=0; % select 1 if you would like to compute statistics along the simulated path

%Calculating Euler Equation Errors
[intertemporal_errors,intratemporal_errors]=sub_foc_errors(grid_k,g_k,g_c,g_l,lambda,n_k,n_z,y,transition);

% Computes the average of the Euler Equation errors 
[meanerror1,meanerror2,maxerror1,maxerror2]=sub_simul(intertemporal_errors,intratemporal_errors,grid_k,g_k,g_l,g_c,transitioncdf,k_ss,y,do_stat);

%----------------------------------------------------------------
% 6. Plot policy functions and the value function
%---------------------------------------------------------------- 
figure;
plot(grid_k,v);
title('Value function')
figure;
plot(grid_k,grid_k,grid_k,g_k);
title('Decision rules for capital');
figure;
plot(grid_k,g_c);
title('Decision rules for consumption');
figure;
plot(grid_k,g_l);
title('Decision rules for labor supply');
